# Copyright (C) 2014  Leo Singer
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
GCN Notice types, from <http://gcn.gsfc.nasa.gov/filtering.html>.
"""
__author__ = "Leo Singer <leo.singer@ligo.org>"


GRB_COORDS = 1
TEST_COORDS = 2
IM_ALIVE = 3
KILL_SOCKET = 4
MAXBC = 11
BRAD_COORDS = 21
GRB_FINAL = 22
HUNTS_SRC = 24
ALEXIS_SRC = 25
XTE_PCA_ALERT = 26
XTE_PCA_SRC = 27
XTE_ASM_ALERT = 28
XTE_ASM_SRC = 29
COMPTEL_SRC = 30
IPN_RAW = 31
IPN_SEG = 32
SAX_WFC_ALERT = 33
SAX_WFC_SRC = 34
SAX_NFI_ALERT = 35
SAX_NFI_SRC = 36
XTE_ASM_TRANS = 37
spare38 = 38
IPN_POS = 39
HETE_ALERT_SRC = 40
HETE_UPDATE_SRC = 41
HETE_FINAL_SRC = 42
HETE_GNDANA_SRC = 43
HETE_TEST = 44
GRB_CNTRPART = 45
SWIFT_TOO_FOM = 46
SWIFT_TOO_SC_SLEW = 47
DOW_TOD = 48
spare50 = 50
INTEGRAL_POINTDIR = 51
INTEGRAL_SPIACS = 52
INTEGRAL_WAKEUP = 53
INTEGRAL_REFINED = 54
INTEGRAL_OFFLINE = 55
INTEGRAL_WEAK = 56
AAVSO = 57
MILAGRO_POS = 58
KONUS_LC = 59
SWIFT_BAT_GRB_ALERT = 60
SWIFT_BAT_GRB_POS_ACK = 61
SWIFT_BAT_GRB_POS_NACK = 62
SWIFT_BAT_GRB_LC = 63
SWIFT_BAT_SCALEDMAP = 64
SWIFT_FOM_OBS = 65
SWIFT_SC_SLEW = 66
SWIFT_XRT_POSITION = 67
SWIFT_XRT_SPECTRUM = 68
SWIFT_XRT_IMAGE = 69
SWIFT_XRT_LC = 70
SWIFT_XRT_CENTROID = 71
SWIFT_UVOT_DBURST = 72
SWIFT_UVOT_FCHART = 73
SWIFT_BAT_GRB_LC_PROC = 76
SWIFT_XRT_SPECTRUM_PROC = 77
SWIFT_XRT_IMAGE_PROC = 78
SWIFT_UVOT_DBURST_PROC = 79
SWIFT_UVOT_FCHART_PROC = 80
SWIFT_UVOT_POS = 81
SWIFT_BAT_GRB_POS_TEST = 82
SWIFT_POINTDIR = 83
SWIFT_BAT_TRANS = 84
SWIFT_XRT_THRESHPIX = 85
SWIFT_XRT_THRESHPIX_PROC = 86
SWIFT_XRT_SPER = 87
SWIFT_XRT_SPER_PROC = 88
SWIFT_UVOT_POS_NACK = 89
SWIFT_BAT_ALARM_SHORT = 90
SWIFT_BAT_ALARM_LONG = 91
SWIFT_UVOT_EMERGENCY = 92
SWIFT_XRT_EMERGENCY = 93
SWIFT_FOM_PPT_ARG_ERR = 94
SWIFT_FOM_SAFE_POINT = 95
SWIFT_FOM_SLEW_ABORT = 96
SWIFT_BAT_QL_POS = 97
SWIFT_BAT_SUB_THRESHOLD = 98
SWIFT_BAT_SLEW_POS = 99
AGILE_GRB_WAKEUP = 100
AGILE_GRB_GROUND = 101
AGILE_GRB_REFINED = 102
AGILE_POINTDIR = 107
AGILE_TRANS = 108
AGILE_GRB_POS_TEST = 109
FERMI_GBM_ALERT = 110
FERMI_GBM_FLT_POS = 111
FERMI_GBM_GND_POS = 112
FERMI_GBM_LC = 113
FERMI_GBM_GND_INTERNAL = 114
FERMI_GBM_FIN_POS = 115
FERMI_GBM_TRANS = 118
FERMI_GBM_POS_TEST = 119
FERMI_LAT_POS_INI = 120
FERMI_LAT_POS_UPD = 121
FERMI_LAT_POS_DIAG = 122
FERMI_LAT_TRANS = 123
FERMI_LAT_POS_TEST = 124
FERMI_LAT_MONITOR = 125
FERMI_SC_SLEW = 126
FERMI_LAT_GND = 127
FERMI_LAT_OFFLINE = 128
FERMI_POINTDIR = 129
SIMBADNED = 130
PIOTS_OT_POS = 131
KAIT_SN = 132
SWIFT_BAT_MONITOR = 133
MAXI_UNKNOWN = 134
MAXI_KNOWN = 135
MAXI_TEST = 136
OGLE = 137
CBAT = 138
MOA = 139
SWIFT_BAT_SUBSUB = 140
SWIFT_BAT_KNOWN_SRC = 141
VOE_11_IM_ALIVE = 142
VOE_20_IM_ALIVE = 143
SUZAKU_LC = 148
LVC_PRELIMINARY = 150
LVC_INITIAL = 151
LVC_UPDATE = 152
LVC_TEST = 153
LVC_COUNTERPART = 154
AMON_ICECUBE_COINC = 157
AMON_ICECUBE_HESE = 158
CALET_GBM_FLT_LC = 160
CALET_GBM_GND_LC = 161
AMON_ICECUBE_EHE = 169



import threading
import gcn
import gcn.voeventclient
import gcn.handlers
import socket
import time

# Set up logger
import logging
logging.basicConfig(level=logging.INFO)
log = logging.getLogger()

payloads = ['''<?xml version = '1.0' encoding = 'UTF-8'?>
<voe:VOEvent
      ivorn="ivo://nasa.gsfc.gcn/Fermi#GBM_Flt_Pos_2011-09-04T03:54:36.02_336801278_45-956"
      role="observation" version="1.1"
      xmlns:voe="http://www.ivoa.net/xml/VOEvent/v1.1"
      xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
      xmlns:xlink="http://www.w3.org/1999/xlink"
      xsi:schemaLocation="http://www.ivoa.net/xml/VOEvent/v1.1 http://www.ivoa.net/xml/VOEvent/VOEvent-v1.1.xsd" >
  <Who>
    <AuthorIVORN>ivo://nasa.gsfc.tan/gcn</AuthorIVORN>
    <Author>
      <shortName>Fermi (via VO-GCN)</shortName>
      <contactName>Julie Mcenery</contactName>
      <contactPhone>+1-301-286-1632</contactPhone>
      <contactEmail>Julie.E.McEnery@nasa.gov</contactEmail>
    </Author>
    <Date>2011-09-04T03:54:51</Date>
    <Description>This VOEvent message was created with GCN VOE version: 1.4 04jun11</Description>
  </Who>
  <What>
    <Param name="Packet_Type"    value="111" />
    <Param name="Pkt_Ser_Num"    value="10" />
    <Param name="TrigID"         value="336801278" ucd="meta.id" />
    <Param name="Sequence_Num"   value="45" ucd="meta.id.part" />
    <Param name="Burst_TJD"      value="15808" unit="days" ucd="time" />
    <Param name="Burst_SOD"      value="14076.02" unit="sec" ucd="time" />
    <Param name="Burst_Inten"    value="117" unit="cts" ucd="phot.count" />
    <Param name="Trig_Timescale" value="8.192" unit="sec" ucd="time.interval" />
    <Param name="Data_Timescale" value="8.192" unit="sec" ucd="time.interval" />
    <Param name="Data_Signif"    value="8.00" unit="sigma" ucd="stat.snr" />
    <Param name="Phi"            value="55.00" unit="deg" ucd="pos.az.azi" />
    <Param name="Theta"          value="65.00" unit="deg" ucd="pos.az.zd" />
    <Param name="SC_Long"        value="115.18" unit="deg" ucd="pos.earth.lon" />
    <Param name="SC_Lat"         value="0.00" unit="deg" ucd="pos.earth.lat" />
    <Param name="Algorithm"             value="3" unit="dn" />
    <Param name="Most_Likely_Index"     value="4" unit="dn" />
    <Param name="Most_Likely_Prob"      value="93" />
    <Param name="Sec_Most_Likely_Index" value="8" unit="dn" />
    <Param name="Sec_Most_Likely_Prob"  value="5" />
    <Param name="Hardness_Ratio"        value="1.39" ucd="arith.ratio" />
    <Param name="Trigger_ID"            value="0x0" />
    <Param name="Misc_flags"            value="0x1000000" />
    <Group name="Trigger_ID" >
      <Param name="Def_NOT_a_GRB"         value="false" />
      <Param name="Target_in_Blk_Catalog" value="false" />
      <Param name="Spatial_Prox_Match"    value="false" />
      <Param name="Temporal_Prox_Match"   value="false" />
      <Param name="Test_Submission"       value="false" />
    </Group>
    <Group name="Misc_Flags" >
      <Param name="Values_Out_of_Range"   value="false" />
      <Param name="Delayed_Transmission"  value="true" />
      <Param name="Flt_Generated"         value="true" />
      <Param name="Gnd_Generated"         value="false" />
    </Group>
    <Param name="Coords_Type"   value="1" unit="dn" />
    <Param name="Coords_String" value="source_object" />
    <Group name="Obs_Support_Info" >
      <Description>The Sun and Moon values are valid at the time the VOEvent XML message was created.</Description>
      <Param name="Sun_RA"        value="162.74" unit="deg" ucd="pos.eq.ra" />
      <Param name="Sun_Dec"       value="7.33" unit="deg" ucd="pos.eq.dec" />
      <Param name="Sun_Distance"  value="48.74" unit="deg" ucd="pos.angDistance" />
      <Param name="Sun_Hr_Angle"  value="-2.03" unit="hr" />
      <Param name="Moon_RA"       value="241.71" unit="deg" ucd="pos.eq.ra" />
      <Param name="Moon_Dec"      value="-22.42" unit="deg" ucd="pos.eq.dec" />
      <Param name="MOON_Distance" value="43.86" unit="deg" ucd="pos.angDistance" />
      <Param name="Moon_Illum"    value="43.70" unit="%" ucd="arith.ratio" />
      <Param name="Galactic_Long" value="303.07" unit="deg" ucd="pos.galactic.lon" />
      <Param name="Galactic_Lat"  value="31.12" unit="deg" ucd="pos.galactic.lat" />
      <Param name="Ecliptic_Long" value="204.91" unit="deg" ucd="pos.ecliptic.lon" />
      <Param name="Ecliptic_Lat"  value="-24.00" unit="deg" ucd="pos.ecliptic.lat" />
    </Group>
    <Description>The Fermi-GBM location of a transient.</Description>
  </What>
  <WhereWhen>
    <ObsDataLocation xmlns="http://www.ivoa.net/xml/STC/stc-v1.30.xsd">
      <ObservatoryLocation xlink:href="ivo://STClib/Observatories#GEOLUN/" xlink:type="simple" id="GEOLUN" />
      <ObservationLocation>
        <AstroCoordSystem xlink:href="ivo://STClib/CoordSys#UTC-FK5-GEO/" xlink:type="simple" id="FK5-UTC-GEO" />
        <AstroCoords coord_system_id="FK5-UTC-GEO">
          <Time unit="s">
            <TimeInstant>
              <ISOTime>2011-09-04T03:54:36.02</ISOTime>
            </TimeInstant>
          </Time>
          <Position2D unit="deg">
            <Name1>RA</Name1>
            <Name2>Dec</Name2>
            <Value2>
              <C1 pos_unit="deg">193.0000</C1>
              <C2 pos_unit="deg">-31.7500</C2>
            </Value2>
            <Error2Radius>17.4333</Error2Radius>
          </Position2D>
        </AstroCoords>
      </ObservationLocation>
    </ObsDataLocation>
  <Description>The RA,Dec coordinates are of the type: source_object.</Description>
  </WhereWhen>
  <How>
    <Description>Fermi Satellite, GBM Instrument</Description>
    <Reference uri="http://gcn.gsfc.nasa.gov/fermi.html" type="url" />
  </How>
  <Why importance="0.5">
    <Inference probability="0.5">
      <Concept>process.variation.burst;em.gamma</Concept>
    </Inference>
  </Why>
  <Citations>
    <EventIVORN cite="followup">ivo://nasa.gsfc.gcn/Fermi#GBM_Alert_2011-09-04T03:54:36.02_336801278_1-954</EventIVORN>
    <Description>This is an updated position to the original trigger.</Description> 
  </Citations>
  <Description>
  </Description>
</voe:VOEvent>
'''.encode('UTF-8'),
'''<?xml version = '1.0' encoding = 'UTF-8'?>
<voe:VOEvent
      ivorn="ivo://nasa.gsfc.gcn/gcn"
      role="observation" version="1.1"
      xmlns:voe="http://www.ivoa.net/xml/VOEvent/v1.1"
      xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
      xmlns:xlink="http://www.w3.org/1999/xlink"
      xsi:schemaLocation="http://www.ivoa.net/xml/VOEvent/v1.1 http://www.ivoa.net/xml/VOEvent/VOEvent-v1.1.xsd" >
  <What>
    <Param name="Packet_Type" value="4" />
  </What>
</voe:VOEvent>
'''.encode('UTF-8')]


def serve(payloads, host='127.0.0.1', port=8099, retransmit_timeout=0, log=None):
    """Rudimentary GCN server, for testing purposes. Serves just one connection
    at a time, and repeats the same payloads in order, repeating, for each
    connection."""
    if log is None:
        log = logging.getLogger('gcn.serve')

    sock = socket.socket()
    try:
        sock.bind((host, port))
        log.info("bound to %s:%d", host, port)
        sock.listen(0)
        for i in range(5):
            conn, addr = sock.accept()
            log.info("connected to %s:%d", addr, port)
            try:
                for payload in payloads:
                    time.sleep(retransmit_timeout)
                    gcn.voeventclient._send_packet(conn, payload)
            except socket.error:
                log.exception('error communicating with peer')
            finally:
                try:
                    conn.shutdown(socket.SHUT_RDWR)
                except socket.error:
                    log.exception("could not shut down socket")

                try:
                    conn.close()
                except socket.error:
                    log.exception("could not close socket")
                else:
                    log.info("closed socket")
    finally:
        sock.close()

class MessageCounter(object):
    def __init__(self):
        self.count = 0
    def __call__(self, *args):
        self.count += 1

def test_reconnect_after_kill():
    """Test that the client recovers if the server closes the connection."""
    server_thread = threading.Thread(
        group=None, target=serve, args=(payloads,), kwargs=dict(retransmit_timeout=0.1))
    server_thread.daemon = True
    server_thread.start()

    handler = MessageCounter()

    client_thread = threading.Thread(
        group=None, target=gcn.listen,
        kwargs=dict(host='127.0.0.1', max_reconnect_timeout=4,
            handler=gcn.handlers.include_notice_types(111)(handler)))
    client_thread.daemon = True
    client_thread.start()

    time.sleep(5)
    assert handler.count == 5



# Copyright (C) 2014  Leo Singer
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

# Bootstrap setuptools installation.
try:
    import pkg_resources
    pkg_resources.require("setuptools >= 0.7")
except:
    from ez_setup import use_setuptools
    use_setuptools()

from setuptools import setup
import gcn
import sys

install_requires = []
python_version_tuple = sys.version_info[:2]
if python_version_tuple == (2, 6) or python_version_tuple == (3, 1):
    install_requires += ['lxml']

setup(
    name='pygcn',
    version='0.1.9',
    author='Leo Singer',
    author_email='leo.singer@ligo.org',
    description=gcn.__doc__.splitlines()[1],
    long_description=gcn.__doc__,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.1',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet',
        'Topic :: Scientific/Engineering :: Astronomy'
    ],
    license='GPL-2+',
    url='https://github.com/lpsinger/pygcn',
    packages=['gcn'],
    scripts=['bin/pygcn-listen', 'bin/pygcn-serve'],
    test_suite='nose.collector',
    tests_require=['nose'],
    install_requires=install_requires
)